/**************************************\
* bmp2nes.c                            *
* Windows .bmp to NES .chr converter   *
*                                      ********************************\
* Copyright 2000 Damian Yerrick                                        *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
* Or view the License online at http://www.gnu.org/copyleft/gpl.html   *
*                                                                      *
* Damian Yerrick's World Wide Web pages are located at                 *
*   http://come.to/yerrick                                             *
*                                                                      *
\**********************************************************************/

#include <allegro.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef ALLEGRO_DJGPP
/* leave most of Allegro out of the statically linked exe */

BEGIN_JOYSTICK_DRIVER_LIST
END_JOYSTICK_DRIVER_LIST
BEGIN_DIGI_DRIVER_LIST
END_DIGI_DRIVER_LIST
BEGIN_MIDI_DRIVER_LIST
END_MIDI_DRIVER_LIST

BEGIN_GFX_DRIVER_LIST
END_GFX_DRIVER_LIST
BEGIN_COLOR_DEPTH_LIST
  COLOR_DEPTH_8
END_COLOR_DEPTH_LIST

#endif

/**************************************\
* BMP to CHR conversion                *
*                                      *
\**************************************/

int convert_BMP2CHR(const char *path)
{
  BITMAP *bmp = load_bitmap(path, NULL);
  char newpath[1024];

  replace_extension(newpath, path, "chr", 1024);

  if(bmp == 0)
  {
    fprintf(stderr, "could not load bitmap %s\n", path);
    return 1;
  }
  else
  {
    unsigned int height = (bmp->h + 7) / 8 * 8;
    unsigned int width = (bmp->w + 7) / 8 * 8;
    unsigned x, y, cx, cy, cz, curByte;
    FILE *fp = fopen(newpath, "wb");

    if(fp == 0)
    {
      fprintf(stderr, "could not open destination file\n");
      destroy_bitmap(bmp);
      return 1;
    }
    else
    {
      for(y = 0; y < height; y += 8) /* for each row of tiles */
      {
        for(x = 0; x < width; x += 8) /* for each tile */
        {
          /* for the Game Boy version, interchange the cz and cy loops */

          for(cz = 0; cz < 2; cz++) /* for each plane */
          {
            for(cy = y; cy < y + 8; cy++) /* for each byte */
            {
              curByte = 0;
              for(cx = x; cx < x + 8; cx++) /* for each bit */
              {
                curByte <<= 1;
                curByte |= (getpixel(bmp, cx, cy) >> cz) & 1;
              }
              fputc(curByte, fp);
            }
          }
        }
      }
      fclose(fp);
    }
    destroy_bitmap(bmp);
  }
  return 0;
}

int main(int argc, const char **argv)
{
  int arg = 1;

  /* using allegro only for bmp file reading */
  install_allegro(SYSTEM_NONE, &errno, (int (*)(void (*)(void)))atexit);

  if(argc < 2)
  {
    fputs("syntax: bmp2nes file.bmp\n", stderr);
    return EXIT_FAILURE;
  }
  while(arg < argc)
  {
    if(convert_BMP2CHR(argv[arg++]))
      return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}
